@@ -13,7 +13,7 @@ module.exports = {
   },
   overrides: [
     {
-      files: ["acorn/src/bin/*.js", "bin/generate-identifier-regex.js"],
+      files: ["acorn/src/bin/*.js", "bin/generate-identifier-regex.js", "bin/generate-unicode-script-values.js"],
       rules: {
         "no-console": "off"
       }
@@ -21,6 +21,8 @@ module.exports = {
   ],
   plugins: ["eslint-plugin-import"],
   rules: {
+    "no-unreachable-loop": "off",
+    "no-empty": "off",
     curly: "off",
     eqeqeq: ["error", "always", {null: "ignore"}],
     indent: [
