@@ -54,8 +54,19 @@ lp.parseStatement = function() {
     this.expect(tt.parenL)
     if (this.tok.type === tt.semi) return this.parseFor(node, null)
     let isLet = this.toks.isLet()
-    if (isLet || this.tok.type === tt._var || this.tok.type === tt._const) {
-      let init = this.parseVar(this.startNode(), true, isLet ? "let" : this.tok.value)
+    let isAwaitUsing = this.toks.isAwaitUsing(true)
+    let isUsing = !isAwaitUsing && this.toks.isUsing(true)
+
+    if (isLet || this.tok.type === tt._var || this.tok.type === tt._const || isUsing || isAwaitUsing) {
+      let kind = isLet ? "let" : isUsing ? "using" : isAwaitUsing ? "await using" : this.tok.value
+      let init = this.startNode()
+      if (isUsing || isAwaitUsing) {
+        if (isAwaitUsing) this.next()
+        this.parseVar(init, true, kind)
+      } else {
+        init = this.parseVar(init, true, kind)
+      }
+
       if (init.declarations.length === 1 && (this.tok.type === tt._in || this.isContextual("of"))) {
         if (this.options.ecmaVersion >= 9 && this.tok.type !== tt._in) {
           node.await = isAwait
@@ -196,6 +207,16 @@ lp.parseStatement = function() {
       this.next()
       return this.parseFunction(node, true, true)
     }
+
+    if (this.toks.isUsing(false)) {
+      return this.parseVar(node, false, "using")
+    }
+
+    if (this.toks.isAwaitUsing(false)) {
+      this.next()
+      return this.parseVar(node, false, "await using")
+    }
+
     let expr = this.parseExpression()
     if (isDummy(expr)) {
       this.next()
@@ -461,6 +482,8 @@ lp.parseExport = function() {
       }
     }
     node.source = this.eatContextual("from") ? this.parseExprAtom() : this.dummyString()
+    if (this.options.ecmaVersion >= 16)
+      node.attributes = this.parseWithClause()
     this.semicolon()
     return this.finishNode(node, "ExportAllDeclaration")
   }
@@ -488,6 +511,8 @@ lp.parseExport = function() {
     node.declaration = null
     node.specifiers = this.parseExportSpecifierList()
     node.source = this.eatContextual("from") ? this.parseExprAtom() : null
+    if (this.options.ecmaVersion >= 16)
+      node.attributes = this.parseWithClause()
     this.semicolon()
   }
   return this.finishNode(node, "ExportNamedDeclaration")
@@ -511,6 +536,8 @@ lp.parseImport = function() {
     node.source = this.eatContextual("from") && this.tok.type === tt.string ? this.parseExprAtom() : this.dummyString()
     if (elt) node.specifiers.unshift(elt)
   }
+  if (this.options.ecmaVersion >= 16)
+    node.attributes = this.parseWithClause()
   this.semicolon()
   return this.finishNode(node, "ImportDeclaration")
 }
@@ -548,6 +575,37 @@ lp.parseImportSpecifiers = function() {
   return elts
 }
 
+lp.parseWithClause = function() {
+  let nodes = []
+  if (!this.eat(tt._with)) {
+    return nodes
+  }
+
+  let indent = this.curIndent, line = this.curLineStart, continuedLine = this.nextLineStart
+  this.pushCx()
+  this.eat(tt.braceL)
+  if (this.curLineStart > continuedLine) continuedLine = this.curLineStart
+  while (!this.closes(tt.braceR, indent + (this.curLineStart <= continuedLine ? 1 : 0), line)) {
+    const attr = this.startNode()
+    attr.key = this.tok.type === tt.string ? this.parseExprAtom() : this.parseIdent()
+    if (this.eat(tt.colon)) {
+      if (this.tok.type === tt.string)
+        attr.value = this.parseExprAtom()
+      else attr.value = this.dummyString()
+    } else {
+      if (isDummy(attr.key)) break
+      if (this.tok.type === tt.string)
+        attr.value = this.parseExprAtom()
+      else break
+    }
+    nodes.push(this.finishNode(attr, "ImportAttribute"))
+    this.eat(tt.comma)
+  }
+  this.eat(tt.braceR)
+  this.popCx()
+  return nodes
+}
+
 lp.parseExportSpecifierList = function() {
   let elts = []
   let indent = this.curIndent, line = this.curLineStart, continuedLine = this.nextLineStart
