@@ -1,4 +1,4 @@
-// AST walker module for Mozilla Parser API compatible trees
+// AST walker module for ESTree compatible trees
 
 // A simple walk is one where you simply specify callbacks to be
 // called on specific nodes. The last two arguments are optional. A
@@ -8,7 +8,7 @@
 //         Expression: function(node) { ... }
 //     });
 //
-// to do something with all expressions. All Parser API node types
+// to do something with all expressions. All ESTree node types
 // can be used to identify node types, as well as Expression and
 // Statement, which denote categories of nodes.
 //
@@ -19,9 +19,9 @@
 export function simple(node, visitors, baseVisitor, state, override) {
   if (!baseVisitor) baseVisitor = base
   ;(function c(node, st, override) {
-    let type = override || node.type, found = visitors[type]
+    let type = override || node.type
     baseVisitor[type](node, st, c)
-    if (found) found(node, st)
+    if (visitors[type]) visitors[type](node, st)
   })(node, state, override)
 }
 
@@ -32,11 +32,11 @@ export function ancestor(node, visitors, baseVisitor, state, override) {
   let ancestors = []
   if (!baseVisitor) baseVisitor = base
   ;(function c(node, st, override) {
-    let type = override || node.type, found = visitors[type]
+    let type = override || node.type
     let isNew = node !== ancestors[ancestors.length - 1]
     if (isNew) ancestors.push(node)
     baseVisitor[type](node, st, c)
-    if (found) found(node, st || ancestors, ancestors)
+    if (visitors[type]) visitors[type](node, st || ancestors, ancestors)
     if (isNew) ancestors.pop()
   })(node, state, override)
 }
@@ -207,11 +207,7 @@ base.WithStatement = (node, st, c) => {
 }
 base.SwitchStatement = (node, st, c) => {
   c(node.discriminant, st, "Expression")
-  for (let cs of node.cases) {
-    if (cs.test) c(cs.test, st, "Expression")
-    for (let cons of cs.consequent)
-      c(cons, st, "Statement")
-  }
+  for (let cs of node.cases) c(cs, st)
 }
 base.SwitchCase = (node, st, c) => {
   if (node.test) c(node.test, st, "Expression")
@@ -351,19 +347,30 @@ base.ExportNamedDeclaration = base.ExportDefaultDeclaration = (node, st, c) => {
   if (node.declaration)
     c(node.declaration, st, node.type === "ExportNamedDeclaration" || node.declaration.id ? "Statement" : "Expression")
   if (node.source) c(node.source, st, "Expression")
+  if (node.attributes)
+    for (let attr of node.attributes)
+      c(attr, st)
 }
 base.ExportAllDeclaration = (node, st, c) => {
   if (node.exported)
     c(node.exported, st)
   c(node.source, st, "Expression")
+  for (let attr of node.attributes)
+    c(attr, st)
+}
+base.ImportAttribute = (node, st, c) => {
+  c(node.value, st, "Expression")
 }
 base.ImportDeclaration = (node, st, c) => {
   for (let spec of node.specifiers)
     c(spec, st)
   c(node.source, st, "Expression")
+  for (let attr of node.attributes)
+    c(attr, st)
 }
 base.ImportExpression = (node, st, c) => {
   c(node.source, st, "Expression")
+  if (node.options) c(node.options, st, "Expression")
 }
 base.ImportSpecifier = base.ImportDefaultSpecifier = base.ImportNamespaceSpecifier = base.Identifier = base.PrivateIdentifier = base.Literal = ignore
 
