@@ -4,7 +4,7 @@ import {lineBreak, skipWhiteSpace} from "./whitespace.js"
 import {isIdentifierStart, isIdentifierChar, keywordRelationalOperator} from "./identifier.js"
 import {hasOwn, loneSurrogate} from "./util.js"
 import {DestructuringErrors} from "./parseutil.js"
-import {functionFlags, SCOPE_SIMPLE_CATCH, BIND_SIMPLE_CATCH, BIND_LEXICAL, BIND_VAR, BIND_FUNCTION, SCOPE_CLASS_STATIC_BLOCK, SCOPE_SUPER} from "./scopeflags.js"
+import {functionFlags, SCOPE_SIMPLE_CATCH, BIND_SIMPLE_CATCH, BIND_LEXICAL, BIND_VAR, BIND_FUNCTION, SCOPE_CLASS_STATIC_BLOCK, SCOPE_SUPER, SCOPE_CLASS_FIELD_INIT} from "./scopeflags.js"
 
 const pp = Parser.prototype
 
@@ -42,10 +42,10 @@ pp.isLet = function(context) {
   // Statement) is allowed here. If context is not empty then only a Statement
   // is allowed. However, `let [` is an explicit negative lookahead for
   // ExpressionStatement, so special-case it first.
-  if (nextCh === 91 || nextCh === 92 || nextCh > 0xd7ff && nextCh < 0xdc00) return true // '[', '/', astral
+  if (nextCh === 91 || nextCh === 92) return true // '[', '\'
   if (context) return false
 
-  if (nextCh === 123) return true // '{'
+  if (nextCh === 123 || nextCh > 0xd7ff && nextCh < 0xdc00) return true // '{', astral
   if (isIdentifierStart(nextCh, true)) {
     let pos = next + 1
     while (isIdentifierChar(nextCh = this.input.charCodeAt(pos), true)) ++pos
@@ -72,6 +72,49 @@ pp.isAsyncFunction = function() {
      !(isIdentifierChar(after = this.input.charCodeAt(next + 8)) || after > 0xd7ff && after < 0xdc00))
 }
 
+pp.isUsingKeyword = function(isAwaitUsing, isFor) {
+  if (this.options.ecmaVersion < 17 || !this.isContextual(isAwaitUsing ? "await" : "using"))
+    return false
+
+  skipWhiteSpace.lastIndex = this.pos
+  let skip = skipWhiteSpace.exec(this.input)
+  let next = this.pos + skip[0].length
+
+  if (lineBreak.test(this.input.slice(this.pos, next))) return false
+
+  if (isAwaitUsing) {
+    let awaitEndPos = next + 5 /* await */, after
+    if (this.input.slice(next, awaitEndPos) !== "using" ||
+      awaitEndPos === this.input.length ||
+      isIdentifierChar(after = this.input.charCodeAt(awaitEndPos)) ||
+      (after > 0xd7ff && after < 0xdc00)
+    ) return false
+
+    skipWhiteSpace.lastIndex = awaitEndPos
+    let skipAfterUsing = skipWhiteSpace.exec(this.input)
+    if (skipAfterUsing && lineBreak.test(this.input.slice(awaitEndPos, awaitEndPos + skipAfterUsing[0].length))) return false
+  }
+
+  if (isFor) {
+    let ofEndPos = next + 2 /* of */, after
+    if (this.input.slice(next, ofEndPos) === "of") {
+      if (ofEndPos === this.input.length ||
+        (!isIdentifierChar(after = this.input.charCodeAt(ofEndPos)) && !(after > 0xd7ff && after < 0xdc00))) return false
+    }
+  }
+
+  let ch = this.input.charCodeAt(next)
+  return isIdentifierStart(ch, true) || ch === 92 // '\'
+}
+
+pp.isAwaitUsing = function(isFor) {
+  return this.isUsingKeyword(true, isFor)
+}
+
+pp.isUsing = function(isFor) {
+  return this.isUsingKeyword(false, isFor)
+}
+
 // Parse a single statement.
 //
 // If expecting a statement and finding a slash operator, parse a
@@ -148,6 +191,23 @@ pp.parseStatement = function(context, topLevel, exports) {
       return this.parseFunctionStatement(node, true, !context)
     }
 
+    let usingKind = this.isAwaitUsing(false) ? "await using" : this.isUsing(false) ? "using" : null
+    if (usingKind) {
+      if (topLevel && this.options.sourceType === "script") {
+        this.raise(this.start, "Using declaration cannot appear in the top level when source type is `script`")
+      }
+      if (usingKind === "await using") {
+        if (!this.canAwait) {
+          this.raise(this.start, "Await using cannot appear outside of async function")
+        }
+        this.next()
+      }
+      this.next()
+      this.parseVar(node, false, usingKind)
+      this.semicolon()
+      return this.finishNode(node, "VariableDeclaration")
+    }
+
     let maybeName = this.value, expr = this.parseExpression()
     if (starttype === tt.name && expr.type === "Identifier" && this.eat(tt.colon))
       return this.parseLabeledStatement(node, maybeName, expr, context)
@@ -223,25 +283,32 @@ pp.parseForStatement = function(node) {
     this.next()
     this.parseVar(init, true, kind)
     this.finishNode(init, "VariableDeclaration")
-    if ((this.type === tt._in || (this.options.ecmaVersion >= 6 && this.isContextual("of"))) && init.declarations.length === 1) {
-      if (this.options.ecmaVersion >= 9) {
-        if (this.type === tt._in) {
-          if (awaitAt > -1) this.unexpected(awaitAt)
-        } else node.await = awaitAt > -1
-      }
-      return this.parseForIn(node, init)
-    }
-    if (awaitAt > -1) this.unexpected(awaitAt)
-    return this.parseFor(node, init)
+    return this.parseForAfterInit(node, init, awaitAt)
   }
   let startsWithLet = this.isContextual("let"), isForOf = false
+
+  let usingKind = this.isUsing(true) ? "using" : this.isAwaitUsing(true) ? "await using" : null
+  if (usingKind) {
+    let init = this.startNode()
+    this.next()
+    if (usingKind === "await using") this.next()
+    this.parseVar(init, true, usingKind)
+    this.finishNode(init, "VariableDeclaration")
+    return this.parseForAfterInit(node, init, awaitAt)
+  }
+  let containsEsc = this.containsEsc
   let refDestructuringErrors = new DestructuringErrors
-  let init = this.parseExpression(awaitAt > -1 ? "await" : true, refDestructuringErrors)
+  let initPos = this.start
+  let init = awaitAt > -1
+    ? this.parseExprSubscripts(refDestructuringErrors, "await")
+    : this.parseExpression(true, refDestructuringErrors)
   if (this.type === tt._in || (isForOf = this.options.ecmaVersion >= 6 && this.isContextual("of"))) {
-    if (this.options.ecmaVersion >= 9) {
-      if (this.type === tt._in) {
-        if (awaitAt > -1) this.unexpected(awaitAt)
-      } else node.await = awaitAt > -1
+    if (awaitAt > -1) { // implies `ecmaVersion >= 9` (see declaration of awaitAt)
+      if (this.type === tt._in) this.unexpected(awaitAt)
+      node.await = true
+    } else if (isForOf && this.options.ecmaVersion >= 8) {
+      if (init.start === initPos && !containsEsc && init.type === "Identifier" && init.name === "async") this.unexpected()
+      else if (this.options.ecmaVersion >= 9) node.await = false
     }
     if (startsWithLet && isForOf) this.raise(init.start, "The left-hand side of a for-of loop may not start with 'let'.")
     this.toAssignable(init, false, refDestructuringErrors)
@@ -254,6 +321,20 @@ pp.parseForStatement = function(node) {
   return this.parseFor(node, init)
 }
 
+// Helper method to parse for loop after variable initialization
+pp.parseForAfterInit = function(node, init, awaitAt) {
+  if ((this.type === tt._in || (this.options.ecmaVersion >= 6 && this.isContextual("of"))) && init.declarations.length === 1) {
+    if (this.options.ecmaVersion >= 9) {
+      if (this.type === tt._in) {
+        if (awaitAt > -1) this.unexpected(awaitAt)
+      } else node.await = awaitAt > -1
+    }
+    return this.parseForIn(node, init)
+  }
+  if (awaitAt > -1) this.unexpected(awaitAt)
+  return this.parseFor(node, init)
+}
+
 pp.parseFunctionStatement = function(node, isAsync, declarationPosition) {
   this.next()
   return this.parseFunction(node, FUNC_STATEMENT | (declarationPosition ? 0 : FUNC_HANGING_STATEMENT), false, isAsync)
@@ -335,6 +416,16 @@ pp.parseThrowStatement = function(node) {
 
 const empty = []
 
+pp.parseCatchClauseParam = function() {
+  const param = this.parseBindingAtom()
+  let simple = param.type === "Identifier"
+  this.enterScope(simple ? SCOPE_SIMPLE_CATCH : 0)
+  this.checkLValPattern(param, simple ? BIND_SIMPLE_CATCH : BIND_LEXICAL)
+  this.expect(tt.parenR)
+
+  return param
+}
+
 pp.parseTryStatement = function(node) {
   this.next()
   node.block = this.parseBlock()
@@ -343,11 +434,7 @@ pp.parseTryStatement = function(node) {
     let clause = this.startNode()
     this.next()
     if (this.eat(tt.parenL)) {
-      clause.param = this.parseBindingAtom()
-      let simple = clause.param.type === "Identifier"
-      this.enterScope(simple ? SCOPE_SIMPLE_CATCH : 0)
-      this.checkLValPattern(clause.param, simple ? BIND_SIMPLE_CATCH : BIND_LEXICAL)
-      this.expect(tt.parenR)
+      clause.param = this.parseCatchClauseParam()
     } else {
       if (this.options.ecmaVersion < 10) this.unexpected()
       clause.param = null
@@ -363,9 +450,9 @@ pp.parseTryStatement = function(node) {
   return this.finishNode(node, "TryStatement")
 }
 
-pp.parseVarStatement = function(node, kind) {
+pp.parseVarStatement = function(node, kind, allowMissingInitializer) {
   this.next()
-  this.parseVar(node, false, kind)
+  this.parseVar(node, false, kind, allowMissingInitializer)
   this.semicolon()
   return this.finishNode(node, "VariableDeclaration")
 }
@@ -489,7 +576,7 @@ pp.parseForIn = function(node, init) {
 
 // Parse a list of variable declarations.
 
-pp.parseVar = function(node, isFor, kind) {
+pp.parseVar = function(node, isFor, kind, allowMissingInitializer) {
   node.declarations = []
   node.kind = kind
   for (;;) {
@@ -497,9 +584,11 @@ pp.parseVar = function(node, isFor, kind) {
     this.parseVarId(decl, kind)
     if (this.eat(tt.eq)) {
       decl.init = this.parseMaybeAssign(isFor)
-    } else if (kind === "const" && !(this.type === tt._in || (this.options.ecmaVersion >= 6 && this.isContextual("of")))) {
+    } else if (!allowMissingInitializer && kind === "const" && !(this.type === tt._in || (this.options.ecmaVersion >= 6 && this.isContextual("of")))) {
       this.unexpected()
-    } else if (decl.id.type !== "Identifier" && !(isFor && (this.type === tt._in || this.isContextual("of")))) {
+    } else if (!allowMissingInitializer && (kind === "using" || kind === "await using") && this.options.ecmaVersion >= 17 && this.type !== tt._in && !this.isContextual("of")) {
+      this.raise(this.lastTokEnd, `Missing initializer in ${kind} declaration`)
+    } else if (!allowMissingInitializer && decl.id.type !== "Identifier" && !(isFor && (this.type === tt._in || this.isContextual("of")))) {
       this.raise(this.lastTokEnd, "Complex binding patterns require an initialization value")
     } else {
       decl.init = null
@@ -511,7 +600,10 @@ pp.parseVar = function(node, isFor, kind) {
 }
 
 pp.parseVarId = function(decl, kind) {
-  decl.id = this.parseBindingAtom()
+  decl.id = kind === "using" || kind === "await using"
+    ? this.parseIdent()
+    : this.parseBindingAtom()
+
   this.checkLValPattern(decl.id, kind === "var" ? BIND_VAR : BIND_LEXICAL, false)
 }
 
@@ -588,7 +680,7 @@ pp.parseClass = function(node, isStatement) {
     if (element) {
       classBody.body.push(element)
       if (element.type === "MethodDefinition" && element.kind === "constructor") {
-        if (hadConstructor) this.raise(element.start, "Duplicate constructor in the same class")
+        if (hadConstructor) this.raiseRecoverable(element.start, "Duplicate constructor in the same class")
         hadConstructor = true
       } else if (element.key && element.key.type === "PrivateIdentifier" && isPrivateNameConflicted(privateNameMap, element)) {
         this.raiseRecoverable(element.key.start, `Identifier '#${element.key.name}' has already been declared`)
@@ -730,11 +822,9 @@ pp.parseClassField = function(field) {
 
   if (this.eat(tt.eq)) {
     // To raise SyntaxError if 'arguments' exists in the initializer.
-    const scope = this.currentThisScope()
-    const inClassFieldInit = scope.inClassFieldInit
-    scope.inClassFieldInit = true
+    this.enterScope(SCOPE_CLASS_FIELD_INIT | SCOPE_SUPER)
     field.value = this.parseMaybeAssign()
-    scope.inClassFieldInit = inClassFieldInit
+    this.exitScope()
   } else {
     field.value = null
   }
@@ -773,7 +863,7 @@ pp.parseClassId = function(node, isStatement) {
 }
 
 pp.parseClassSuper = function(node) {
-  node.superClass = this.eat(tt._extends) ? this.parseExprSubscripts(false) : null
+  node.superClass = this.eat(tt._extends) ? this.parseExprSubscripts(null, false) : null
 }
 
 pp.enterClassBody = function() {
@@ -784,6 +874,7 @@ pp.enterClassBody = function() {
 
 pp.exitClassBody = function() {
   const {declared, used} = this.privateNameStack.pop()
+  if (!this.options.checkPrivateFields) return
   const len = this.privateNameStack.length
   const parent = len === 0 ? null : this.privateNameStack[len - 1]
   for (let i = 0; i < used.length; ++i) {
@@ -834,56 +925,54 @@ function checkKeyName(node, name) {
 
 // Parses module export declaration.
 
+pp.parseExportAllDeclaration = function(node, exports) {
+  if (this.options.ecmaVersion >= 11) {
+    if (this.eatContextual("as")) {
+      node.exported = this.parseModuleExportName()
+      this.checkExport(exports, node.exported, this.lastTokStart)
+    } else {
+      node.exported = null
+    }
+  }
+  this.expectContextual("from")
+  if (this.type !== tt.string) this.unexpected()
+  node.source = this.parseExprAtom()
+  if (this.options.ecmaVersion >= 16)
+    node.attributes = this.parseWithClause()
+  this.semicolon()
+  return this.finishNode(node, "ExportAllDeclaration")
+}
+
 pp.parseExport = function(node, exports) {
   this.next()
   // export * from '...'
   if (this.eat(tt.star)) {
-    if (this.options.ecmaVersion >= 11) {
-      if (this.eatContextual("as")) {
-        node.exported = this.parseModuleExportName()
-        this.checkExport(exports, node.exported, this.lastTokStart)
-      } else {
-        node.exported = null
-      }
-    }
-    this.expectContextual("from")
-    if (this.type !== tt.string) this.unexpected()
-    node.source = this.parseExprAtom()
-    this.semicolon()
-    return this.finishNode(node, "ExportAllDeclaration")
+    return this.parseExportAllDeclaration(node, exports)
   }
   if (this.eat(tt._default)) { // export default ...
     this.checkExport(exports, "default", this.lastTokStart)
-    let isAsync
-    if (this.type === tt._function || (isAsync = this.isAsyncFunction())) {
-      let fNode = this.startNode()
-      this.next()
-      if (isAsync) this.next()
-      node.declaration = this.parseFunction(fNode, FUNC_STATEMENT | FUNC_NULLABLE_ID, false, isAsync)
-    } else if (this.type === tt._class) {
-      let cNode = this.startNode()
-      node.declaration = this.parseClass(cNode, "nullableID")
-    } else {
-      node.declaration = this.parseMaybeAssign()
-      this.semicolon()
-    }
+    node.declaration = this.parseExportDefaultDeclaration()
     return this.finishNode(node, "ExportDefaultDeclaration")
   }
   // export var|const|let|function|class ...
   if (this.shouldParseExportStatement()) {
-    node.declaration = this.parseStatement(null)
+    node.declaration = this.parseExportDeclaration(node)
     if (node.declaration.type === "VariableDeclaration")
       this.checkVariableExport(exports, node.declaration.declarations)
     else
       this.checkExport(exports, node.declaration.id, node.declaration.id.start)
     node.specifiers = []
     node.source = null
+    if (this.options.ecmaVersion >= 16)
+      node.attributes = []
   } else { // export { x, y as z } [from '...']
     node.declaration = null
     node.specifiers = this.parseExportSpecifiers(exports)
     if (this.eatContextual("from")) {
       if (this.type !== tt.string) this.unexpected()
       node.source = this.parseExprAtom()
+      if (this.options.ecmaVersion >= 16)
+        node.attributes = this.parseWithClause()
     } else {
       for (let spec of node.specifiers) {
         // check for keywords used as local names
@@ -897,12 +986,35 @@ pp.parseExport = function(node, exports) {
       }
 
       node.source = null
+      if (this.options.ecmaVersion >= 16)
+        node.attributes = []
     }
     this.semicolon()
   }
   return this.finishNode(node, "ExportNamedDeclaration")
 }
 
+pp.parseExportDeclaration = function(node) {
+  return this.parseStatement(null)
+}
+
+pp.parseExportDefaultDeclaration = function() {
+  let isAsync
+  if (this.type === tt._function || (isAsync = this.isAsyncFunction())) {
+    let fNode = this.startNode()
+    this.next()
+    if (isAsync) this.next()
+    return this.parseFunction(fNode, FUNC_STATEMENT | FUNC_NULLABLE_ID, false, isAsync)
+  } else if (this.type === tt._class) {
+    let cNode = this.startNode()
+    return this.parseClass(cNode, "nullableID")
+  } else {
+    let declaration = this.parseMaybeAssign()
+    this.semicolon()
+    return declaration
+  }
+}
+
 pp.checkExport = function(exports, name, pos) {
   if (!exports) return
   if (typeof name !== "string")
@@ -929,8 +1041,6 @@ pp.checkPatternExport = function(exports, pat) {
     this.checkPatternExport(exports, pat.left)
   else if (type === "RestElement")
     this.checkPatternExport(exports, pat.argument)
-  else if (type === "ParenthesizedExpression")
-    this.checkPatternExport(exports, pat.expression)
 }
 
 pp.checkVariableExport = function(exports, decls) {
@@ -950,6 +1060,20 @@ pp.shouldParseExportStatement = function() {
 
 // Parses a comma-separated list of module exports.
 
+pp.parseExportSpecifier = function(exports) {
+  let node = this.startNode()
+  node.local = this.parseModuleExportName()
+
+  node.exported = this.eatContextual("as") ? this.parseModuleExportName() : node.local
+  this.checkExport(
+    exports,
+    node.exported,
+    node.exported.start
+  )
+
+  return this.finishNode(node, "ExportSpecifier")
+}
+
 pp.parseExportSpecifiers = function(exports) {
   let nodes = [], first = true
   // export { x, y as z } [from '...']
@@ -960,15 +1084,7 @@ pp.parseExportSpecifiers = function(exports) {
       if (this.afterTrailingComma(tt.braceR)) break
     } else first = false
 
-    let node = this.startNode()
-    node.local = this.parseModuleExportName()
-    node.exported = this.eatContextual("as") ? this.parseModuleExportName() : node.local
-    this.checkExport(
-      exports,
-      node.exported,
-      node.exported.start
-    )
-    nodes.push(this.finishNode(node, "ExportSpecifier"))
+    nodes.push(this.parseExportSpecifier(exports))
   }
   return nodes
 }
@@ -977,6 +1093,7 @@ pp.parseExportSpecifiers = function(exports) {
 
 pp.parseImport = function(node) {
   this.next()
+
   // import '...'
   if (this.type === tt.string) {
     node.specifiers = empty
@@ -986,29 +1103,54 @@ pp.parseImport = function(node) {
     this.expectContextual("from")
     node.source = this.type === tt.string ? this.parseExprAtom() : this.unexpected()
   }
+  if (this.options.ecmaVersion >= 16)
+    node.attributes = this.parseWithClause()
   this.semicolon()
   return this.finishNode(node, "ImportDeclaration")
 }
 
 // Parses a comma-separated list of module imports.
 
+pp.parseImportSpecifier = function() {
+  let node = this.startNode()
+  node.imported = this.parseModuleExportName()
+
+  if (this.eatContextual("as")) {
+    node.local = this.parseIdent()
+  } else {
+    this.checkUnreserved(node.imported)
+    node.local = node.imported
+  }
+  this.checkLValSimple(node.local, BIND_LEXICAL)
+
+  return this.finishNode(node, "ImportSpecifier")
+}
+
+pp.parseImportDefaultSpecifier = function() {
+  // import defaultObj, { x, y as z } from '...'
+  let node = this.startNode()
+  node.local = this.parseIdent()
+  this.checkLValSimple(node.local, BIND_LEXICAL)
+  return this.finishNode(node, "ImportDefaultSpecifier")
+}
+
+pp.parseImportNamespaceSpecifier = function() {
+  let node = this.startNode()
+  this.next()
+  this.expectContextual("as")
+  node.local = this.parseIdent()
+  this.checkLValSimple(node.local, BIND_LEXICAL)
+  return this.finishNode(node, "ImportNamespaceSpecifier")
+}
+
 pp.parseImportSpecifiers = function() {
   let nodes = [], first = true
   if (this.type === tt.name) {
-    // import defaultObj, { x, y as z } from '...'
-    let node = this.startNode()
-    node.local = this.parseIdent()
-    this.checkLValSimple(node.local, BIND_LEXICAL)
-    nodes.push(this.finishNode(node, "ImportDefaultSpecifier"))
+    nodes.push(this.parseImportDefaultSpecifier())
     if (!this.eat(tt.comma)) return nodes
   }
   if (this.type === tt.star) {
-    let node = this.startNode()
-    this.next()
-    this.expectContextual("as")
-    node.local = this.parseIdent()
-    this.checkLValSimple(node.local, BIND_LEXICAL)
-    nodes.push(this.finishNode(node, "ImportNamespaceSpecifier"))
+    nodes.push(this.parseImportNamespaceSpecifier())
     return nodes
   }
   this.expect(tt.braceL)
@@ -1018,20 +1160,46 @@ pp.parseImportSpecifiers = function() {
       if (this.afterTrailingComma(tt.braceR)) break
     } else first = false
 
-    let node = this.startNode()
-    node.imported = this.parseModuleExportName()
-    if (this.eatContextual("as")) {
-      node.local = this.parseIdent()
-    } else {
-      this.checkUnreserved(node.imported)
-      node.local = node.imported
-    }
-    this.checkLValSimple(node.local, BIND_LEXICAL)
-    nodes.push(this.finishNode(node, "ImportSpecifier"))
+    nodes.push(this.parseImportSpecifier())
   }
   return nodes
 }
 
+pp.parseWithClause = function() {
+  let nodes = []
+  if (!this.eat(tt._with)) {
+    return nodes
+  }
+  this.expect(tt.braceL)
+  const attributeKeys = {}
+  let first = true
+  while (!this.eat(tt.braceR)) {
+    if (!first) {
+      this.expect(tt.comma)
+      if (this.afterTrailingComma(tt.braceR)) break
+    } else first = false
+
+    const attr = this.parseImportAttribute()
+    const keyName = attr.key.type === "Identifier" ? attr.key.name : attr.key.value
+    if (hasOwn(attributeKeys, keyName))
+      this.raiseRecoverable(attr.key.start, "Duplicate attribute key '" + keyName + "'")
+    attributeKeys[keyName] = true
+    nodes.push(attr)
+  }
+  return nodes
+}
+
+pp.parseImportAttribute = function() {
+  const node = this.startNode()
+  node.key = this.type === tt.string ? this.parseExprAtom() : this.parseIdent(this.options.allowReserved !== "never")
+  this.expect(tt.colon)
+  if (this.type !== tt.string) {
+    this.unexpected()
+  }
+  node.value = this.parseExprAtom()
+  return this.finishNode(node, "ImportAttribute")
+}
+
 pp.parseModuleExportName = function() {
   if (this.options.ecmaVersion >= 13 && this.type === tt.string) {
     const stringLiteral = this.parseLiteral(this.value)
