@@ -14,10 +14,13 @@
   require("./tests-regexp-2018.js");
   require("./tests-regexp-2020.js");
   require("./tests-regexp-2022.js");
+  require("./tests-regexp-2024.js");
+  require("./tests-regexp-2025.js");
   require("./tests-json-superset.js");
   require("./tests-optional-catch-binding.js");
   require("./tests-bigint.js");
   require("./tests-dynamic-import.js");
+  require("./tests-export-named.js");
   require("./tests-export-all-as-ns-from-source.js");
   require("./tests-import-meta.js");
   require("./tests-nullish-coalescing.js");
@@ -26,6 +29,8 @@
   require("./tests-numeric-separators.js");
   require("./tests-class-features-2022.js");
   require("./tests-module-string-names.js");
+  require("./tests-import-attributes.js");
+  require("./tests-using.js");
   var acorn = require("../acorn")
   var acorn_loose = require("../acorn-loose")
 
