@@ -2,6 +2,7 @@
 if (typeof exports !== "undefined") {
   var driver = require("./driver.js");
   var test = driver.test;
+  var testFail = driver.testFail;
 }
 
 //------------------------------------------------------------------------
@@ -550,7 +551,6 @@ test("({ wrap() { \"use strict\"; foo } })", {
               "end": 7,
               "name": "wrap"
             },
-            "kind": "init",
             "value": {
               "type": "FunctionExpression",
               "start": 7,
@@ -591,7 +591,8 @@ test("({ wrap() { \"use strict\"; foo } })", {
                   }
                 ]
               }
-            }
+            },
+            "kind": "init"
           }
         ]
       },
@@ -1384,3 +1385,8 @@ test("(a = () => { \"use strict\"; foo }) => { \"use strict\" }", {
     }
   ]
 }, { ecmaVersion: 6 })
+
+testFail(
+  "function invalid() { \"\\7\\\n\"; \"use strict\"; }",
+  "Octal literal in strict mode (1:22)",
+  { ecmaVersion: 6 })
